; scope4433.s, Osciloskop
; Uros Platise (c) 2001
;
; Simple Protocol: RS-232 Commands, 38400
;
; Command       Hex             Description
;-------------------------------------------------------------------------
; Time Scale:   1x              x = {abbb}   b: adc prescaller value
;                                            a: extra slow mode
; Phase Shift:	20
; Trigger Dly:  30,H,L                   (H,L): 16 bit delay (unit: sample)
; Trigger Lvl:  4x,L            x = {000a}   a: rising/falling edge, 
;                                            L: 0-255 (0-5 V)
; Measure:      8x              x = {aaaa}   a: delay in 20 ms
;                                               returns 100 8 bits results
; Abort:        90

#arch AT90S4433
#include "avr.inc"

;----------------------------------------------------------------------
; Declarations
;----------------------------------------------------------------------

; Temporary Registers
#define Tmp		r16
#define adcL		r17
#define adcH		r18
#define Loop		r19
#define Command		Loop
#define Tmp2		Loop

; Storage Trigger Delay Regs
#define trig_dlyL	r1
#define trig_dlyH	r2

; Active Trigger Delay Regs
#define trigA_dly	X
#define trigA_dlyL	XL
#define trigA_dlyH	XH

; Buf Counter
#define bufCnt		r20

; Trigger Noisce Canceller
#define trigNC		r21

; Extra delay after measurement is complete to perform system loop
#define extra_dly	r22

; Memory Write Pointer
#define memp		Z
#define mempL		ZL
#define mempH		ZH

; Commands
#define SP_TIMESCALE	0x10
#define SP_PHASESHIFT	0x20
#define SP_TRIGDELAY	0x30
#define SP_TRIGLEVEL	0x40
#define SP_MEASURE	0x80
#define SP_ABORT	0x90

; Leds
#define TRIG_LED_OFF	sbi PORTB,PB0
#define TRIG_LED_ON	cbi PORTB,PB0
#define SKIP_TRIGLED_ON	sbic PORTB,PB0
#define SKIP_TRIGLED_OFF sbis PORTB,PB0


;----------------------------------------------------------------------
; Vectors
;----------------------------------------------------------------------

	seg abs=0 flash.vectors	
	rjmp	reset
	reti			; irq0
	reti			; irq1
	rjmp	ac_trigger	; timer1 capture
	reti			; timer1 compare
	reti			; timer1 ovw
	reti			; timer0 ovw
	reti			; spi 
	rjmp	uartrx		; uart rx
	reti			; uart udr
	reti			; uart tx
	rjmp	rdst_adc	; adc
	reti			; eeprom
	reti			; analog comparator
	

;----------------------------------------------------------------------
; Program
;----------------------------------------------------------------------

	seg flash.code	
reset:	
	; Stack
	ldi	Tmp, stack
	out	SPL, Tmp
	
	; CPU IDLE mode
	ldi	Tmp, BV(SE)
	out	MCUCR, Tmp
				
	; ADC Base Config
	ldi	Tmp, BV(ADEN)+BV(ADSC)+BV(ADFR)+BV(ADPS2)+BV(ADPS1)
	out	ADCSR, Tmp
	
	; Channel ADC0
	ldi	Tmp, 0
	out	ADMUX, Tmp
	
	; Setup Analog Comparator
	ldi	Tmp, BV(ACIC)
	out	ACSR, Tmp
	
	; And Input Capture Logic
	ldi	Tmp, BV(TICIE1)
	out	TIMSK, Tmp
	
	; Setup PWM output - Trigger Level - 8 bit, max possible freq
	ldi	Tmp, BV(PWM10)+BV(COM11)
	out	TCCR1A, Tmp
	ldi	Tmp, BV(CS10)+BV(ICNC1)
	out	TCCR1B, Tmp
	clr	Tmp
	out	OCR1AH, Tmp
	ldi	Tmp, 0x80
	out	OCR1AL, Tmp
	ldi	Tmp, BV(PB1)+BV(PB0)
	out	DDRB, Tmp
	TRIG_LED_OFF
	
	; RS-232 38400
        clr     Tmp
        out     UBRRH, Tmp
        ldi     Tmp, 12
        out     UBRR, Tmp
        ldi     Tmp, BV(RXEN)+BV(TXEN)+BV(RXCIE)
        out     UCR, Tmp
	
	; Init Data Memory Pointer
	ldi	mempL, datamem
	clr	mempH

	; Init Preferences	
	clr	extra_dly
	ldi	trigA_dlyL,100
	ldi	trigA_dlyH,0
	mov	trig_dlyL, trigA_dlyL
	mov	trig_dlyH, trigA_dlyH	
	
	; Debug
	ldi	Tmp, BV(PD5)
	out	DDRD, Tmp
	out	PORTD, Tmp
#define DB_ON	cbi PORTD, PD5
#define DB_OFF	sbi PORTD, PD5
	
	; All Functions are driven by interrupts!
	sei
loop:	
	rjmp	loop

;----------------------------------------------------------------------
; Read and Store ADC Result
;----------------------------------------------------------------------
rdst_adc:
	tst	bufCnt
	breq	bufCnt_zero
	dec	bufCnt
bufCnt_zero:	

	; read 6 cycles
	in	adcL, ADCL
	in	adcH, ADCH
	lsr	adcH
	ror	adcL
	lsr	adcH
	ror	adcL

	; store result 5 cycles
	st	memp+, adcL
	cpi	mempL, datamem_end
	brne	do_not_reset_memp
	ldi	mempL, datamem
do_not_reset_memp:

	; Was trigger captured already? (trig_cap == 0, when yes)
	SKIP_TRIGLED_OFF
	rjmp	exit_rdst_adc

	; decrement number of samples counter after triggering
	sbiw	trigA_dlyL, 1
	brne	exit_rdst_adc
	
	; clear trigger, stop A/D converter
	cbi	ADCSR, ADIE
	
	; add extra delay for about 1.25 ms
loop_exd:
	tst	extra_dly
	breq	loop_exe
	clr	trigA_dlyL
	ldi	trigA_dlyH, 10
loop_exdi:	
	sbiw	trigA_dlyL,1 
	brne	loop_exdi
	dec	extra_dly
	rjmp	loop_exd
loop_exe:
	
	; read results
	ldi	Loop, 100	
read_next:	
	ld	Tmp, memp+
	out	UDR, Tmp

	; wait until sent
wait_until_sent:
	sbis	USR, UDRE
	rjmp	wait_until_sent
			
	cpi	mempL, datamem_end
	brne	do_not_reset_read_memp
	ldi	mempL, datamem
do_not_reset_read_memp:
	dec	Loop
	brne	read_next
	
	
	; exit	
exit_rdst_adc:
	reti
	

;----------------------------------------------------------------------
; Analog Comparator Trigger; notice that trigger was captured
;----------------------------------------------------------------------
ac_trigger:
	tst	bufCnt
	brne	preread_buffer_not_full
	TRIG_LED_OFF
preread_buffer_not_full:
	reti


;----------------------------------------------------------------------
; UART Rx Interface
;----------------------------------------------------------------------	
uartrx:
	in	Tmp, UDR
	mov	Command, Tmp
	andi	Command, 0xF0	; command
	andi	Tmp, 0x0F	; paramater
	
	cpi	Command, SP_MEASURE
	breq	sp_measure
	cpi	Command, SP_TIMESCALE
	breq	sp_timescale
	cpi	Command, SP_PHASESHIFT
	breq	sp_phaseshift
	cpi	Command, SP_TRIGLEVEL
	breq	sp_triglevel
	cpi	Command, SP_TRIGDELAY
	breq	sp_trigdelay
	cpi	Command, SP_ABORT
	breq	sp_abort
	reti
	
	; MEASURE COMMAND; enable ADC scanning and comparator triggering
sp_measure:
	TRIG_LED_ON
	mov	trigA_dlyH, trig_dlyH
	mov	trigA_dlyL, trig_dlyL
	mov	extra_dly, Tmp
	swap	extra_dly		; swap bits in upper area
	ldi	bufCnt, 100
	sbi	ADCSR, ADIE
	reti	
	
	; TIME SCALE (change A/D prescaler value)
sp_timescale:
	in	Tmp2, ADCSR
	andi	Tmp2, 0xF8
	or	Tmp2, Tmp
	out	ADCSR, Tmp2
	reti
	
	; PHASE SHIFT for A/D 2 CKs (sample period)
sp_phaseshift:
	in	Tmp2, ADCSR
	mov	extra_dly,Tmp2		; this var. is not used at that time
	ori	extra_dly,BV(ADPS0)+BV(ADPS1)	; set presc = 3
	out	ADCSR, extra_dly
	nop nop nop nop
	out	ADCSR, Tmp2
sp_phaseshift_exit:
	reti
	
	; EDGE SELECTION and TRIGGER LEVEL
sp_triglevel:
	ldi	Tmp2, BV(CS10)+BV(ICNC1)
	sbrc	Tmp, 0
	ori	Tmp2, BV(ICES1)
	out	TCCR1B, Tmp2
	sbrc	Tmp, 0	

	cbi	UCR, RXCIE
	rcall	wait_rx_data
	clr	Tmp
	out	OCR1AH, Tmp
	in	Tmp, UDR
	out	OCR1AL, Tmp
	sbi	UCR, RXCIE	
	reti
		
	; TRIGGER DELAY
sp_trigdelay:
	cbi	UCR, RXCIE
	rcall	wait_rx_data
	in	trig_dlyH, UDR
	rcall	wait_rx_data
	in	trig_dlyL, UDR
	sbi	UCR, RXCIE	
	reti
	
	; ABORT MEASUREMENT
sp_abort:
	TRIG_LED_OFF
	cbi	ADCSR, ADIE
	reti
	
	
	; WAIT FOR NEXT RS-232 BYTE
wait_rx_data:
wait_rxc:	
	sbis	USR, RXC
	rjmp	wait_rxc
	ret

;----------------------------------------------------------------------
; Data
;----------------------------------------------------------------------
	
	seg eram.stack
datamem:ds.b	100
datamem_end:
	
	ds.b	7
stack:	ds.b	1
